#!/bin/bash

if [ $# != 1 ]; then
	echo "modis_calc_index.sh usage: [target year]"
	exit 1
fi

if [ -z "$GISBASE" ] ; then
    echo "ERROR: You must be in GRASS GIS to run this program." 1>&2
    exit 1
fi

if [ -z "$R_BIN" ]; then
	ENVFILE=`dirname $0`/modis_setenv.txt
	echo $ENVFILE
	if [ ! -e $ENVFILE ]; then
		g.message -e "modis_setenv.txt can not be found. You must put modis_setenv.txt in the same directory as this script."
		exit 1
	fi
	source $ENVFILE
fi

YEAR=$1

#
# check if index layer is already created.
#
LISTNUM=`g.mlist type=rast pattern="interp_${PRODUCT_NAME}.A${YEAR}*_ndvi" | wc -l`
if [ $LISTNUM -eq 0 ]; then 
	g.message -e "There is no index layers."
	g.message -e "Please execute modis_calc_index.sh first to create index layers."
	exit 1
fi

mkdir temp
cd temp

#
# NDVIAVE and order matrix
#

for I in `g.mlist type=rast pattern="interp_${PRODUCT_NAME}.A${YEAR}*_ndvi"`; do

	echo $I
	r.out.gdal input=$I output=${I}.tif type=Float32 createopt="TFW=YES" 2>/dev/null
	if [ $? -ne 0 ]; then
		exit 1
	fi

done

rm -f *.xml

$R_BIN --vanilla --slave << EOF
	# 3GBɐݒ
	memory.limit(size=3590)

	calcNDVI <- function( ndvis )
	{
		s.ndvi <- ndvis[!is.nan(ndvis)]

		r.ndviave <- mean( sort( s.ndvi )[1:3] )
	}

	applyCBind <- function( vec )
	{
		if ( length( vec ) < 2 ) stop( "Length of vec is 1" )
		if ( length( vec ) == 2 ) cbind( get( vec[1] ), get( vec[2] ) )
		else
			cbind( get( vec[1] ), applyCBind( vec[2:length(vec)] ) )
	}

	order.pix <- function( vec )
	{
		if ( length(vec) == 3 )
			out.order <- order( vec )
		else
			out.order <- order( vec )[-1:-(length(vec)-3)]
		return(out.order)
	}

	library(rgdal)

	ndvis <- c()
	n = 0
	for ( L in list.files( pattern="*_ndvi.tif" ) )
	{
		SPLIT <- unlist( strsplit( L, ".", fixed = TRUE ) )
		OBNAME <- paste( SPLIT[1], SPLIT[2] )
		print( OBNAME )
		DATA <- readGDAL( L )
		MATRIX <- as.matrix( DATA )
		assign( OBNAME, MATRIX )
		ndvis <- c( ndvis, OBNAME )
		if ( n == 0 )
		{
			xcoord <- coordinates(DATA)[,1]
			ycoord <- coordinates(DATA)[,2]
			pjstr <- proj4string(DATA)
			n = 1
		}
		rm(DATA)
	}

	per <- length( ndvis )
	ALL <- applyCBind( ndvis )
	rc <- dim( get( ndvis[1] ) )
	dim( ALL ) <- c( rc, per )
	dim( ALL )

	out.ndviave <- matrix( nrow=rc[1], ncol=rc[2] )
	ndvi <- double( per )
	out.order <- array( dim=c( rc,3 ) )

#	for ( R in 1:nrow( ALL ) )
#	{
#		for ( C in 1:ncol( ALL ) )
#		{
#			if ( per == 3 )
#				out.order[R,C,] <- order( ALL[R,C,] )
#			else
#				out.order[R,C,] <- order( ALL[R,C,] )[-1:-(length(ALL[R,C,])-3)]
#
#			out.ndviave[R,C] <- calcNDVI( ALL[R,C,] )
#		}
#	}

	out.order <- apply( ALL, c(1,2), order.pix )
	out.ndviave <- apply( ALL, c(1,2), calcNDVI )

	save( out.order, file="order.dat" )
	d <- list( x=xcoord, y=ycoord, z=out.ndviave )
	gt <- GridTopology( cellcentre.offset=c(min(d\$x), max(d\$y) ),
		cellsize=c(diff(d\$x[1:2]), -1*diff(d\$x[1:2])), cells.dim=dim(d\$z) )
	sgf <- SpatialGridDataFrame( gt, data.frame(depth=as.vector(d\$z[,ncol(d\$z):1])), proj4string=pjstr )
	writeGDAL( sgf, "ndviave.tif" )

EOF

gdalwarp ndviave.tif ndviave_w.tif
r.in.gdal --overwrite input="ndviave_w.tif" output="agg_ndviave"

#
# BANDAVE
#

for (( B = 1; B <= 7; B++ )); do

	echo "band${B}"

	for I in `g.mlist pattern="interp_${PRODUCT_NAME}.A${YEAR}*_band${B}"`; do

		echo $I
		r.out.gdal input=$I output=${I}.tif type=Float32 createopt="TFW=YES" 2>/dev/null
		if [ $? -ne 0 ]; then
			exit 1
		fi

	done

done

rm -f *.xml

$R_BIN --vanilla --slave << EOF
	# 3GBɐݒ
	memory.limit(size=3590)

	calcAve <- function( bands, order )
	{
		s.bands <- bands[!is.nan(bands)]
		r.bandave <- mean( c( s.bands[order[1]], s.bands[order[2]], s.bands[order[3]] ) )
	}

	applyCBind <- function( vec )
	{
		if ( length( vec ) < 2 ) stop( "Length of vec is 1" )
		if ( length( vec ) == 2 ) cbind( get( vec[1] ), get( vec[2] ) )
		else
			cbind( get( vec[1] ), applyCBind( vec[2:length(vec)] ) )
	}

	library(rgdal)

	load( file="order.dat" )

	for ( B in 1:7 )
	{
		bands <- c()
		n = 0
		for ( L in list.files( pattern=paste( "*_band", B, ".tif", sep="" ) ) )
		{
			SPLIT <- unlist( strsplit( L, ".", fixed = TRUE ) )
			OBNAME <- paste( SPLIT[1], SPLIT[2] )
			print( OBNAME )
			DATA <- readGDAL( L )
			MATRIX <- as.matrix( DATA )
			assign( OBNAME, MATRIX )
			bands <- c( bands, OBNAME )
			if ( n == 0 )
			{
				xcoord <- coordinates(DATA)[,1]
				ycoord <- coordinates(DATA)[,2]
				pjstr <- proj4string(DATA)
				n = 1
			}
			rm(DATA)
		}

		per <- length( bands )
		ALL <- applyCBind( bands )
		rc <- dim( get( bands[1] ) )
		dim( ALL ) <- c( rc, per )

		out.bandave <- matrix( nrow=rc[1], ncol=rc[2] )

		for ( R in 1:nrow( ALL ) )
		{
			for ( C in 1:ncol( ALL ) )
			{
				out.bandave[R,C] <- calcAve( ALL[R,C,], out.order[,R,C] )
			}
		}

		d <- list( x=xcoord, y=ycoord, z=out.bandave )
		gt <- GridTopology( cellcentre.offset=c(min(d\$x), max(d\$y) ),
			cellsize=c(diff(d\$x[1:2]), -1*diff(d\$x[1:2])), cells.dim=dim(d\$z) )
		sgf <- SpatialGridDataFrame( gt, data.frame(depth=as.vector(d\$z[,ncol(d\$z):1])), proj4string=pjstr )
		writeGDAL( sgf, paste( "band", B, ".tif", sep="" ) )

		rm( ALL )
		rm( out.bandave )
		rm( d )
		rm( gt )
		rm( sgf )
	}

EOF

for (( B = 1; B <= 7; B++ )); do
	gdalwarp band${B}.tif band${B}_w.tif
	r.in.gdal --overwrite input="band${B}_w.tif" output="agg_band${B}"
done

rm -f *.xml

#
# NDSIMIN
#


for I in `g.mlist pattern="interp_${PRODUCT_NAME}.A${YEAR}*_ndsi"`; do

	echo $I
	r.out.gdal input=$I output=${I}.tif type=Float32 createopt="TFW=YES" 2>/dev/null
	if [ $? -ne 0 ]; then
		exit 1
	fi

done

$R_BIN --vanilla --slave << EOF
	# 3GBɐݒ
	memory.limit(size=3590)

	calcMin <- function( bands, order )
	{
		s.bands <- bands[!is.nan(bands)]
		r.bandave <- min( c( s.bands[order[1]], s.bands[order[2]], s.bands[order[3]] ) )
	}

	applyCBind <- function( vec )
	{
		if ( length( vec ) < 2 ) stop( "Length of vec is 1" )
		if ( length( vec ) == 2 ) cbind( get( vec[1] ), get( vec[2] ) )
		else
			cbind( get( vec[1] ), applyCBind( vec[2:length(vec)] ) )
	}

	library(rgdal)

	load( file="order.dat" )

	ndsis <- c()
	n = 0
	for ( L in list.files( pattern="*_ndsi.tif" ) )
	{
		SPLIT <- unlist( strsplit( L, ".", fixed = TRUE ) )
		OBNAME <- paste( SPLIT[1], SPLIT[2] )
		print( OBNAME )
		DATA <- readGDAL( L )
		MATRIX <- as.matrix( DATA )
		assign( OBNAME, MATRIX )
		ndsis <- c( ndsis, OBNAME )
		if ( n == 0 )
		{
			xcoord <- coordinates(DATA)[,1]
			ycoord <- coordinates(DATA)[,2]
			pjstr <- proj4string(DATA)
			n = 1
		}
		rm(DATA)
	}

	per <- length( ndsis )
	ALL <- applyCBind( ndsis )
	rc <- dim( get( ndsis[1] ) )
	dim( ALL ) <- c( rc, per )
	dim( ALL )

	out.ndsimin <- matrix( nrow=rc[1], ncol=rc[2] )

	for ( R in 1:nrow( ALL ) )
	{
		for ( C in 1:ncol( ALL ) )
		{
			out.ndsimin[R,C] <- calcMin( ALL[R,C,], out.order[,R,C] )
		}
	}

	d <- list( x=xcoord, y=ycoord, z=out.ndsimin )
	gt <- GridTopology( cellcentre.offset=c(min(d\$x), max(d\$y) ),
		cellsize=c(diff(d\$x[1:2]), -1*diff(d\$x[1:2])), cells.dim=dim(d\$z) )
	sgf <- SpatialGridDataFrame( gt, data.frame(depth=as.vector(d\$z[,ncol(d\$z):1])), proj4string=pjstr )
	writeGDAL( sgf, "ndsimin.tif" )

EOF

gdalwarp ndsimin.tif ndsimin_w.tif
r.in.gdal --overwrite input="ndsimin_w.tif" output="agg_ndsimin"


#
# SIAVE
#


for I in `g.mlist pattern="interp_${PRODUCT_NAME}.A${YEAR}*_si"`; do

	echo $I
	r.out.gdal input=$I output=${I}.tif type=Float32 createopt="TFW=YES" 2>/dev/null
	if [ $? -ne 0 ]; then
		exit 1
	fi

done

rm -f *.xml

$R_BIN --vanilla --slave << EOF
	# 3GBɐݒ
	memory.limit(size=3590)

	calcAve <- function( bands, order )
	{
		s.bands <- bands[!is.nan(bands)]
		r.bandave <- mean( c( s.bands[order[1]], s.bands[order[2]], s.bands[order[3]] ) )
	}

	applyCBind <- function( vec )
	{
		if ( length( vec ) < 2 ) stop( "Length of vec is 1" )
		if ( length( vec ) == 2 ) cbind( get( vec[1] ), get( vec[2] ) )
		else
			cbind( get( vec[1] ), applyCBind( vec[2:length(vec)] ) )
	}

	library(rgdal)

	load( file="order.dat" )

	sis <- c()
	n = 0
	for ( L in list.files( pattern="*_si.tif" ) )
	{
		SPLIT <- unlist( strsplit( L, ".", fixed = TRUE ) )
		OBNAME <- paste( SPLIT[1], SPLIT[2] )
		print( OBNAME )
		DATA <- readGDAL( L )
		MATRIX <- as.matrix( DATA )
		assign( OBNAME, MATRIX )
		sis <- c( sis, OBNAME )
		if ( n == 0 )
		{
			xcoord <- coordinates(DATA)[,1]
			ycoord <- coordinates(DATA)[,2]
			pjstr <- proj4string(DATA)
			n = 1
		}
		rm(DATA)
	}

	per <- length( sis )
	ALL <- applyCBind( sis )
	rc <- dim( get( sis[1] ) )
	dim( ALL ) <- c( rc, per )
	dim( ALL )

	out.siave <- matrix( nrow=rc[1], ncol=rc[2] )

	for ( R in 1:nrow( ALL ) )
	{
		for ( C in 1:ncol( ALL ) )
		{
			out.siave[R,C] <- calcAve( ALL[R,C,], out.order[,R,C] )
		}
	}

	d <- list( x=xcoord, y=ycoord, z=out.siave )
	gt <- GridTopology( cellcentre.offset=c(min(d\$x), max(d\$y) ),
		cellsize=c(diff(d\$x[1:2]), -1*diff(d\$x[1:2])), cells.dim=dim(d\$z) )
	sgf <- SpatialGridDataFrame( gt, data.frame(depth=as.vector(d\$z[,ncol(d\$z):1])), proj4string=pjstr )
	writeGDAL( sgf, "siave.tif" )

EOF

gdalwarp siave.tif siave_w.tif
r.in.gdal --overwrite input="siave_w.tif" output="agg_siave"

cd ..
rm -r -f temp
