#!/bin/bash

if [ $# != 0 ] && [ $# != 4 ]; then
	echo "modis_merge.sh usage: [[xmin] [ymin] [xmax] [ymax] (optional)]"
	echo "xmin,ymin,xmax,ymax are the extent of target SRS."
	echo "if not specified, extent of the warped image is calculated automatically."
fi

if [ -z "$R_BIN" ]; then
	ENVFILE=`dirname $0`/modis_setenv.txt
	echo $ENVFILE
	if [ ! -e $ENVFILE ]; then
		echo "ERROR: modis_setenv.txt was not found. You must put modis_setenv.txt in the same directory as this script."
		exit 1
	fi
	source $ENVFILE
fi

if [ $# == 4 ]; then
	OPT_TE="-te $@"
fi

#
# make directory and translate subsets of hdf.
#

for i in `ls *.hdf`; do
	#
	# Interpolate method is "nearest neighbour" for QC image,
	# "lanzos" for other.
	#
	if [ `echo $i | cut -d "." -f 1` = $PRODUCT_NAME ]; then
			INTERP_OPT="-r lanczos"
	fi

	DIRNAME=`echo $i | cut -d "." -f 1-2`
	BASENAME=`basename $i ".hdf"`
	OUTNAME=${DIRNAME}/${BASENAME}_
	mkdir $DIRNAME > /dev/null 2>&1
	gdal_translate -sds -of "GTiff" $i $OUTNAME
	if [ $? -ne 0 ]; then
		exit 1
	fi
	cd $DIRNAME
	for j in `ls --ignore="*.xml" --ignore="*.tif"`
	do
		mv $j ${j}.tif
	done
	cd ..
done

#
# merge images of the same days and warp.
#
for i in `ls -d */`; do
	echo $i
	cd $i
	DIR=`pwd`
	PREFIX=${DIR##*/}
	SUFFIX=1

	# merge
	while true; do

		LIST=`ls *_${SUFFIX}*.tif 2> /dev/null`
	
		if [ ! "$LIST" ] ; then
			break
		fi

		HEAD=`echo $LIST | cut -d " " -f 1`
		NODATAVALUE=`gdalinfo $HEAD | grep "NoData Value" | cut -d "=" -f 2`

		gdal_merge.py -of "GTiff" -o ${PREFIX}_band${SUFFIX}_org.tif -a_nodata ${NODATAVALUE} ${LIST}
		if [ $? -ne 0 ]; then
			exit 1
		fi

		#
		# remove tempolary files.
		#
		rm $LIST

		let SUFFIX="${SUFFIX} + 1"
	done

	#
	# create a MASK image.
	#

	MASKIMG=${PREFIX}_mask
	cp ${PREFIX}_band1_org.tif temp.tif
	BITS=`tiffinfo temp.tif 2> /dev/null |grep "Bits/Sample" | awk '{print $2}'`
	if [ $BITS = 8 ]; then
		NEAR=255
	elif [ $BITS = 16 ]; then
		NEAR=65535
	fi
	nearblack -of "GTiff" -white -near ${NEAR} -o ${MASKIMG}_org.tif temp.tif
	rm temp.tif

	# warp
	SUFFIX=1
	while true; do
		if [ ! -e ${PREFIX}_band${SUFFIX}_org.tif ]; then
			break
		fi

		NODATAVALUE=`gdalinfo ${PREFIX}_band${SUFFIX}_org.tif | grep "NoData Value" | cut -d "=" -f 2`
		gdalwarp -of "GTiff" ${INTERP_OPT} -t_srs EPSG:4326 -dstnodata ${NODATAVALUE} ${OPT_TE} ${PREFIX}_band${SUFFIX}_org.tif ${PREFIX}_band${SUFFIX}_prj.tif
		if [ $? -ne 0 ]; then
			exit 1
		fi
		mv ${PREFIX}_band${SUFFIX}_prj.tif ..

		let SUFFIX="${SUFFIX} + 1"
	done

	#
	# warp mask image.
	#

	gdalwarp -of "GTiff" -t_srs EPSG:4326 -dstnodata 0 ${OPT_TE} ${MASKIMG}_org.tif ${MASKIMG}_prj.tif
	if [ $? -ne 0 ]; then
		exit 1
	fi
	mv ${MASKIMG}_prj.tif ..

	cd ..

	rm -r -f $i
done
